==> ignf_gpf_sdk-0.1.14-py3-none-any.whl: ignf_gpf_sdk/workflow/action/UploadAction.py <==
import time
from typing import Any, Callable, Dict, List, Optional


from ignf_gpf_sdk.Errors import GpfSdkError
from ignf_gpf_sdk.store.Upload import Upload
from ignf_gpf_sdk.io.Dataset import Dataset
from ignf_gpf_sdk.io.Config import Config
from ignf_gpf_sdk.workflow.action.ActionAbstract import ActionAbstract


class UploadAction:
    """Classe permettant d'accompagner la création d'une livraison.

    Attributes:
        __dataset (Dataset): dataset contenant les info de la livraison à créer
        __upload (Optional[Upload]): livraison représentant l'entité créée sur l'entrepôt
        __behavior (str): comportement à adopter si la livraison existe déjà sur l'entrepôt
    """

    BEHAVIOR_STOP = "STOP"
    BEHAVIOR_DELETE = "DELETE"
    BEHAVIOR_CONTINUE = "CONTINUE"

    def __init__(self, dataset: Dataset, behavior: Optional[str] = None) -> None:
        self.__dataset: Dataset = dataset
        self.__upload: Optional[Upload] = None
        # On suit le comportement donnée en paramètre ou à défaut celui de la config
        self.__behavior: str = behavior if behavior is not None else Config().get_str("upload", "behavior_if_exists")

    def run(self, datastore: Optional[str]) -> Upload:
        """Crée la livraison décrite dans le dataset et livre les données avant de
        retourner la livraison créée.

        Args:
            datastore (Optional[str]): id du datastore à utiliser. Si None, le datastore sera récupéré dans la configuration.

        Raises:
            GpfSdkError: levée si création non effectuée

        Returns:
            livraison créée
        """
        Config().om.info("Création et complétion d'une livraison...")
        # Création de la livraison
        self.__create_upload(datastore)
        # Ajout des tags
        self.__add_tags()
        # Ajout des commentaires
        self.__add_comments()
        # Envoie des fichiers de données
        self.__push_data_files()
        # Envoie des fichiers md5
        self.__push_md5_files()
        # Fermeture de la livraison
        self.__close()
        # Affiche et retourne la livraison
        if self.upload is not None:
            # Affichage
            Config().om.info(f"Livraison créée et complétée : {self.__upload}")
            Config().om.info("Création et complétion d'une livraison : terminé")
            # Retour
            return self.upload
        # On ne devrait pas arriver ici...
        raise GpfSdkError("Erreur à la création de la livraison.")

    def __create_upload(self, datastore: Optional[str]) -> None:
        """Crée l'upload après avoir vérifié s'il n'existe pas déjà...

        Args:
            datastore (Optional[str]): id du datastore à utiliser.
        """
        Config().om.info("Création d'une livraison...")
        # On tente de récupérer l'upload
        o_upload = self.find_upload(datastore)
        # S'il n'est pas null
        if o_upload is not None:
            # On sort en erreur si demandé
            if self.__behavior == "STOP":
                raise GpfSdkError(f"Impossible de créer la livraison, une livraison identique {o_upload} existe déjà.")
            # On supprime/recrée la livraison si demandé
            if self.__behavior == "DELETE":
                Config().om.warning(f"Une livraison identique {o_upload} va être supprimée puis recréée...")
                o_upload.api_delete()
                # on en crée une nouvelle (on utilise les champs de "upload_infos" du dataset)
                self.__upload = Upload.api_create(self.__dataset.upload_infos, route_params={"datastore": datastore})
                Config().om.warning(f"Livraison {self.__upload} recréée avec succès.")
            else:
                # Sinon on continue avec cet upload pour le compléter (behavior == CONTINUE)
                # cas livraison fermé : on plante
                if not o_upload.is_open():
                    raise GpfSdkError(f"Impossible de continuer, la livraison {o_upload} est fermée.")
                Config().om.info(f"Livraison identique {o_upload} trouvée, le programme va la reprendre et la compléter.")
                self.__upload = o_upload
        else:
            # Si la livraison est nulle, on en crée une nouvelle (on utilise les champs de "upload_infos" du dataset)
            self.__upload = Upload.api_create(self.__dataset.upload_infos, route_params={"datastore": datastore})
            Config().om.info(f"Livraison {self.__upload['name']} créée avec succès.")

    def __add_tags(self) -> None:
        """Ajoute les tags."""
        if self.__upload is not None and self.__dataset.tags is not None:
            Config().om.info(f"Livraison {self.__upload['name']} : ajout des {len(self.__dataset.tags)} tags...")
            self.__upload.api_add_tags(self.__dataset.tags)
            Config().om.info(f"Livraison {self.__upload['name']} : les {len(self.__dataset.tags)} tags ont été ajoutés avec succès.")

    def __add_comments(self) -> None:
        """Ajoute les commentaires."""
        if self.__upload is not None:
            Config().om.info(f"Livraison {self.__upload['name']} : ajout des {len(self.__dataset.comments)} commentaires...")
            l_actual_comments = [d_comment["text"] for d_comment in self.__upload.api_list_comments() if d_comment]
            for s_comment in self.__dataset.comments:
                if s_comment not in l_actual_comments:
                    self.__upload.api_add_comment({"text": s_comment})
            Config().om.info(f"Livraison {self.__upload['name']} : les {len(self.__dataset.comments)} commentaires ont été ajoutés avec succès.")

    def __push_data_files(self) -> None:
        """Téléverse les fichiers de données (listés dans le dataset)."""
        if self.__upload is not None:
            # Liste les fichiers déjà téléversés sur l'entrepôt et récupère leur taille
            Config().om.info(f"Livraison {self.__upload['name']} : récupération de l'arborescence des données déjà téléversées...")
            l_arborescence = self.__upload.api_tree()
            d_destination_taille = UploadAction.parse_tree(l_arborescence)

            for p_file_path, s_api_path in self.__dataset.data_files.items():
                # Regarde si le fichier du dataset est déjà dans la liste des fichiers téléversés sur l'entrepôt
                # NB: sur l'entrepôt, tous les fichiers "data" sont dans le dossier parent "data" TODO vérifier que c'est toujours le cas !
                s_data_api_path = f"{s_api_path}/{p_file_path.name}"
                Config().om.info(f"Livraison {self.__upload['name']} : livraison de {s_data_api_path}...")
                if s_data_api_path in d_destination_taille:
                    # le fichier est déjà livré, on check sa taille :
                    if d_destination_taille[s_data_api_path] == p_file_path.stat().st_size:
                        # le fichier a été complètement téléversé. On passe au fichier suivant.
                        Config().om.info(f"Livraison {self.__upload['name']} : livraison de {s_data_api_path}: déjà livré")
                        continue

                    # le fichier n'a pas été téléversé en totalité.
                    # Si le mode "Append" n'est pas disponible sur le serveur, il faut supprimer le fichier à moitié téléversé.
                    # Sinon il faudra reprendre le téléversement (!)
                    self.__upload.api_delete_data_file(s_data_api_path)

                # sinon, on doit livrer le fichier
                self.__upload.api_push_data_file(p_file_path, s_api_path)
                Config().om.info(f"Livraison {self.__upload['name']} : livraison de {s_data_api_path}: terminé")
            Config().om.info(f"Livraison {self.__upload}: les {len(self.__dataset.data_files)} fichiers de données ont été ajoutés avec succès.")

    def __push_md5_files(self) -> None:
        """Téléverse les fichiers de clefs (listés dans le dataset)."""
        if self.__upload is not None:
            # Liste les fichiers md5 téléversés sur l'entrepôt et récupère leur taille
            l_arborescence = self.__upload.api_tree()
            d_destination_taille = UploadAction.parse_tree(l_arborescence)

            for p_file_path in self.__dataset.md5_files:
                # Regarde si le fichier du dataset est déjà dans la liste des fichiers téléversés sur l'entrepôt
                # NB: sur l'entrepot, tous les fichiers md5 sont à la racine
                s_api_path = p_file_path.name
                Config().om.info(f"Livraison {self.__upload['name']} : livraison de {s_api_path}...")
                if s_api_path in d_destination_taille:
                    # le fichier est déjà livré, on check sa taille :
                    if d_destination_taille[s_api_path] == p_file_path.stat().st_size:
                        # le fichier a été complètement téléversé. On passe au fichier suivant.
                        Config().om.info(f"Livraison {self.__upload['name']} : livraison de {s_api_path}: déjà livré")
                        continue

                    # le fichier n'a pas été téléversé en totalité.
                    # Si le mode "Append" n'est pas disponible sur le serveur, il faut supprimer le fichier à moitié téléversé.
                    # Sinon il faudra reprendre le téléversement (!)
                    self.__upload.api_delete_data_file(s_api_path)

                # sinon, on doit livrer le fichier
                self.__upload.api_push_md5_file(p_file_path)
                Config().om.info(f"Livraison {self.__upload['name']} : livraison de {s_api_path}: terminé")
            Config().om.info(f"Livraison {self.__upload}: les {len(self.__dataset.md5_files)} fichiers md5 ont été ajoutés avec succès.")

    def __close(self) -> None:
        """Ferme la livraison."""
        if self.__upload is not None:
            Config().om.info(f"Livraison {self.__upload['name']} : fermeture de la livraison...")
            self.__upload.api_close()
            Config().om.info(f"Livraison {self.__upload['name']} : livraison fermée avec succès. La livraison va maintenant être vérifiée par la Géoplateforme.")

    def find_upload(self, datastore: Optional[str]) -> Optional[Upload]:
        """Fonction permettant de lister un éventuel upload déjà existant à partir des critères d'unicité donnés.

        Args:
            datastore (Optional[str]): id du datastore à utiliser.

        Returns:
            None si rien trouvé, sinon l'Upload trouvé
        """
        # Récupération des critères de filtre
        d_infos, d_tags = ActionAbstract.get_filters("upload", self.__dataset.upload_infos, self.__dataset.tags)
        # On peut maintenant filter les upload selon ces critères
        l_uploads = Upload.api_list(infos_filter=d_infos, tags_filter=d_tags, datastore=datastore)
        # S'il y a un ou plusieurs upload, on retourne le 1er :
        if l_uploads:
            return l_uploads[0]
        # sinon on retourne None
        return None

    @property
    def upload(self) -> Optional[Upload]:
        return self.__upload

    @staticmethod
    def monitor_until_end(upload: Upload, callback: Optional[Callable[[str], None]] = None) -> bool:
        """Attend que toute les vérifications liées à la Livraison indiquée
        soient terminées (en erreur ou en succès) avant de rendre la main.

        La fonction callback indiquée est exécutée à chaque vérification en lui passant en paramètre un
        message de suivi du nombre de vérifications par statut.

        Args:
            upload (Upload): Livraison à monitorer
            callback (Optional[Callable[[str], None]]): fonction de callback à exécuter avec le message de suivi.

        Returns:
            True si toutes les vérifications sont ok, sinon False
        """
        i_nb_sec_between_check = Config().get_int("upload", "nb_sec_between_check_updates")
        s_check_message_pattern = Config().get_str("upload", "check_message_pattern")
        b_success: Optional[bool] = None
        Config().om.info(f"Monitoring des vérifications toutes les {i_nb_sec_between_check} secondes...")
        while b_success is None:
            # On récupère les vérifications
            d_checks = upload.api_list_checks()
            # On peut déterminer b_success s'il n'y en a plus en attente et en cours
            if len(d_checks["asked"]) == len(d_checks["in_progress"]) == 0:
                b_success = len(d_checks["failed"]) == 0
            # On affiche un rapport via la fonction de callback précisée
            s_message = s_check_message_pattern.format(
                nb_asked=len(d_checks["asked"]),
                nb_in_progress=len(d_checks["in_progress"]),
                nb_passed=len(d_checks["passed"]),
                nb_failed=len(d_checks["failed"]),
            )
            if callback is not None:
                callback(s_message)
            # Si l'état est toujours indéterminé
            if b_success is None:
                # On attend le temps demandé
                time.sleep(i_nb_sec_between_check)
        # On log le dernier rapport selon l'état et on sort
        if b_success:
            Config().om.info(s_message)
            return True
        Config().om.warning(s_message)
        return False

    @staticmethod
    def parse_tree(tree: List[Dict[str, Any]], prefix: str = "") -> Dict[str, int]:
        """Parse l'arborescence renvoyée par l'API en un dictionnaire associant le chemin de chaque fichier à sa taille.
        L'objectif est de permettre de facilement identifier quels sont les fichiers à (re)livrer.

        Args:
            tree (List[Dict[str, Any]]): arborescence à parser
            prefix (str): pré-fixe du chemin

        Returns:
            liste des fichiers envoyés et leur taille
        """
        # Création du dictionnaire pour stocker les fichiers et leur taille
        d_files: Dict[str, int] = {}
        # Parcours de l'arborescence
        for d_element in tree:
            # On complète le chemin
            if prefix != "":
                s_chemin = f"{prefix}/{d_element['name']}"
            else:
                s_chemin = str(d_element["name"])
            # Fichier ou dossier ?
            if d_element["type"].lower() == "file":
                # Fichier, on l'ajoute à notre dictionnaire
                d_files[s_chemin] = int(d_element["size"])
            elif d_element["type"].lower() == "directory":
                # Dossier, on itère dessus avec le nom du dossier comme préfixe
                d_sub_files = UploadAction.parse_tree(d_element["children"], prefix=s_chemin)
                # On fusionne ces fichiers à notre dict principal
                d_files = {**d_files, **d_sub_files}
            else:
                raise GpfSdkError(f"Type d'élément rencontré dans l'arborescence '{d_element['type']}' non géré. Contacter le support.")
        return d_files

==> ignf_gpf_sdk-0.1.15-py3-none-any.whl: ignf_gpf_sdk/workflow/action/UploadAction.py <==
import time
from typing import Any, Callable, Dict, List, Optional


from ignf_gpf_sdk.Errors import GpfSdkError
from ignf_gpf_sdk.store.Upload import Upload
from ignf_gpf_sdk.io.Dataset import Dataset
from ignf_gpf_sdk.io.Config import Config
from ignf_gpf_sdk.workflow.action.ActionAbstract import ActionAbstract


class UploadAction:
    """Classe permettant d'accompagner la création d'une livraison.

    Attributes:
        __dataset (Dataset): dataset contenant les info de la livraison à créer
        __upload (Optional[Upload]): livraison représentant l'entité créée sur l'entrepôt
        __behavior (str): comportement à adopter si la livraison existe déjà sur l'entrepôt
    """

    BEHAVIOR_STOP = "STOP"
    BEHAVIOR_DELETE = "DELETE"
    BEHAVIOR_CONTINUE = "CONTINUE"

    def __init__(self, dataset: Dataset, behavior: Optional[str] = None) -> None:
        self.__dataset: Dataset = dataset
        self.__upload: Optional[Upload] = None
        # On suit le comportement donnée en paramètre ou à défaut celui de la config
        self.__behavior: str = behavior if behavior is not None else Config().get_str("upload", "behavior_if_exists")

    def run(self, datastore: Optional[str]) -> Upload:
        """Crée la livraison décrite dans le dataset et livre les données avant de
        retourner la livraison créée.

        Args:
            datastore (Optional[str]): id du datastore à utiliser. Si None, le datastore sera récupéré dans la configuration.

        Raises:
            GpfSdkError: levée si création non effectuée

        Returns:
            livraison créée
        """
        Config().om.info("Création et complétion d'une livraison...")
        # Création de la livraison
        self.__create_upload(datastore)

        # Cas livraison fermé = déjà traité : on sort
        if self.upload and not self.upload.is_open():
            return self.upload

        # Ajout des tags
        self.__add_tags()
        # Ajout des commentaires
        self.__add_comments()
        # Envoie des fichiers de données
        self.__push_data_files()
        # Envoie des fichiers md5
        self.__push_md5_files()
        # Fermeture de la livraison
        self.__close()
        # Affiche et retourne la livraison
        if self.upload is not None:
            # Affichage
            Config().om.info(f"Livraison créée et complétée : {self.__upload}")
            Config().om.info("Création et complétion d'une livraison : terminé")
            # Retour
            return self.upload
        # On ne devrait pas arriver ici...
        raise GpfSdkError("Erreur à la création de la livraison.")

    def __create_upload(self, datastore: Optional[str]) -> None:
        """Crée l'upload après avoir vérifié s'il n'existe pas déjà...

        Args:
            datastore (Optional[str]): id du datastore à utiliser.
        """
        Config().om.info("Création d'une livraison...")
        # On tente de récupérer l'upload
        o_upload = self.find_upload(datastore)
        # S'il n'est pas null
        if o_upload is not None:
            # On sort en erreur si demandé
            if self.__behavior == self.BEHAVIOR_STOP:
                raise GpfSdkError(f"Impossible de créer la livraison, une livraison identique {o_upload} existe déjà.")
            # On supprime/recrée la livraison si demandé
            if self.__behavior == self.BEHAVIOR_DELETE:
                Config().om.warning(f"Une livraison identique {o_upload} va être supprimée puis recréée...")
                o_upload.api_delete()
                # on en crée une nouvelle (on utilise les champs de "upload_infos" du dataset)
                self.__upload = Upload.api_create(self.__dataset.upload_infos, route_params={"datastore": datastore})
                Config().om.warning(f"Livraison {self.__upload} recréée avec succès.")
            elif self.__behavior == self.BEHAVIOR_CONTINUE:
                # Sinon on continue avec cet upload pour le compléter (behavior == CONTINUE)
                # cas livraison fermé : message particulier
                if not o_upload.is_open():
                    Config().om.warning(f"Livraison identique {o_upload} trouvée et fermée, cette livraison ne sera pas mise à jour.")
                else:
                    Config().om.info(f"Livraison identique {o_upload} trouvée, le programme va la reprendre et la compléter.")
                self.__upload = o_upload
            else:
                raise GpfSdkError(f"Le comportement {self.__behavior} n'est pas reconnu, l'exécution de traitement est annulée.")
        else:
            # Si la livraison est nulle, on en crée une nouvelle (on utilise les champs de "upload_infos" du dataset)
            self.__upload = Upload.api_create(self.__dataset.upload_infos, route_params={"datastore": datastore})
            Config().om.info(f"Livraison {self.__upload['name']} créée avec succès.")

    def __add_tags(self) -> None:
        """Ajoute les tags."""
        if self.__upload is not None and self.__dataset.tags is not None:
            Config().om.info(f"Livraison {self.__upload['name']} : ajout des {len(self.__dataset.tags)} tags...")
            self.__upload.api_add_tags(self.__dataset.tags)
            Config().om.info(f"Livraison {self.__upload['name']} : les {len(self.__dataset.tags)} tags ont été ajoutés avec succès.")

    def __add_comments(self) -> None:
        """Ajoute les commentaires."""
        if self.__upload is not None:
            Config().om.info(f"Livraison {self.__upload['name']} : ajout des {len(self.__dataset.comments)} commentaires...")
            l_actual_comments = [d_comment["text"] for d_comment in self.__upload.api_list_comments() if d_comment]
            for s_comment in self.__dataset.comments:
                if s_comment not in l_actual_comments:
                    self.__upload.api_add_comment({"text": s_comment})
            Config().om.info(f"Livraison {self.__upload['name']} : les {len(self.__dataset.comments)} commentaires ont été ajoutés avec succès.")

    def __push_data_files(self) -> None:
        """Téléverse les fichiers de données (listés dans le dataset)."""
        if self.__upload is not None:
            # Liste les fichiers déjà téléversés sur l'entrepôt et récupère leur taille
            Config().om.info(f"Livraison {self.__upload['name']} : récupération de l'arborescence des données déjà téléversées...")
            l_arborescence = self.__upload.api_tree()
            d_destination_taille = UploadAction.parse_tree(l_arborescence)

            for p_file_path, s_api_path in self.__dataset.data_files.items():
                # Regarde si le fichier du dataset est déjà dans la liste des fichiers téléversés sur l'entrepôt
                # NB: sur l'entrepôt, tous les fichiers "data" sont dans le dossier parent "data" TODO vérifier que c'est toujours le cas !
                s_data_api_path = f"{s_api_path}/{p_file_path.name}"
                Config().om.info(f"Livraison {self.__upload['name']} : livraison de {s_data_api_path}...")
                if s_data_api_path in d_destination_taille:
                    # le fichier est déjà livré, on check sa taille :
                    if d_destination_taille[s_data_api_path] == p_file_path.stat().st_size:
                        # le fichier a été complètement téléversé. On passe au fichier suivant.
                        Config().om.info(f"Livraison {self.__upload['name']} : livraison de {s_data_api_path}: déjà livré")
                        continue

                    # le fichier n'a pas été téléversé en totalité.
                    # Si le mode "Append" n'est pas disponible sur le serveur, il faut supprimer le fichier à moitié téléversé.
                    # Sinon il faudra reprendre le téléversement (!)
                    self.__upload.api_delete_data_file(s_data_api_path)

                # sinon, on doit livrer le fichier
                self.__upload.api_push_data_file(p_file_path, s_api_path)
                Config().om.info(f"Livraison {self.__upload['name']} : livraison de {s_data_api_path}: terminé")
            Config().om.info(f"Livraison {self.__upload}: les {len(self.__dataset.data_files)} fichiers de données ont été ajoutés avec succès.")

    def __push_md5_files(self) -> None:
        """Téléverse les fichiers de clefs (listés dans le dataset)."""
        if self.__upload is not None:
            # Liste les fichiers md5 téléversés sur l'entrepôt et récupère leur taille
            l_arborescence = self.__upload.api_tree()
            d_destination_taille = UploadAction.parse_tree(l_arborescence)

            for p_file_path in self.__dataset.md5_files:
                # Regarde si le fichier du dataset est déjà dans la liste des fichiers téléversés sur l'entrepôt
                # NB: sur l'entrepot, tous les fichiers md5 sont à la racine
                s_api_path = p_file_path.name
                Config().om.info(f"Livraison {self.__upload['name']} : livraison de {s_api_path}...")
                if s_api_path in d_destination_taille:
                    # le fichier est déjà livré, on check sa taille :
                    if d_destination_taille[s_api_path] == p_file_path.stat().st_size:
                        # le fichier a été complètement téléversé. On passe au fichier suivant.
                        Config().om.info(f"Livraison {self.__upload['name']} : livraison de {s_api_path}: déjà livré")
                        continue

                    # le fichier n'a pas été téléversé en totalité.
                    # Si le mode "Append" n'est pas disponible sur le serveur, il faut supprimer le fichier à moitié téléversé.
                    # Sinon il faudra reprendre le téléversement (!)
                    self.__upload.api_delete_data_file(s_api_path)

                # sinon, on doit livrer le fichier
                self.__upload.api_push_md5_file(p_file_path)
                Config().om.info(f"Livraison {self.__upload['name']} : livraison de {s_api_path}: terminé")
            Config().om.info(f"Livraison {self.__upload}: les {len(self.__dataset.md5_files)} fichiers md5 ont été ajoutés avec succès.")

    def __close(self) -> None:
        """Ferme la livraison."""
        if self.__upload is not None:
            Config().om.info(f"Livraison {self.__upload['name']} : fermeture de la livraison...")
            self.__upload.api_close()
            Config().om.info(f"Livraison {self.__upload['name']} : livraison fermée avec succès. La livraison va maintenant être vérifiée par la Géoplateforme.")

    def find_upload(self, datastore: Optional[str]) -> Optional[Upload]:
        """Fonction permettant de lister un éventuel upload déjà existant à partir des critères d'unicité donnés.

        Args:
            datastore (Optional[str]): id du datastore à utiliser.

        Returns:
            None si rien trouvé, sinon l'Upload trouvé
        """
        # Récupération des critères de filtre
        d_infos, d_tags = ActionAbstract.get_filters("upload", self.__dataset.upload_infos, self.__dataset.tags)
        # On peut maintenant filter les upload selon ces critères
        l_uploads = Upload.api_list(infos_filter=d_infos, tags_filter=d_tags, datastore=datastore)
        # S'il y a un ou plusieurs upload, on retourne le 1er :
        if l_uploads:
            return l_uploads[0]
        # sinon on retourne None
        return None

    @property
    def upload(self) -> Optional[Upload]:
        return self.__upload

    @staticmethod
    def monitor_until_end(upload: Upload, callback: Optional[Callable[[str], None]] = None) -> bool:
        """Attend que toute les vérifications liées à la Livraison indiquée
        soient terminées (en erreur ou en succès) avant de rendre la main.

        La fonction callback indiquée est exécutée à chaque vérification en lui passant en paramètre un
        message de suivi du nombre de vérifications par statut.

        Args:
            upload (Upload): Livraison à monitorer
            callback (Optional[Callable[[str], None]]): fonction de callback à exécuter avec le message de suivi.

        Returns:
            True si toutes les vérifications sont ok, sinon False
        """
        i_nb_sec_between_check = Config().get_int("upload", "nb_sec_between_check_updates")
        s_check_message_pattern = Config().get_str("upload", "check_message_pattern")
        b_success: Optional[bool] = None
        Config().om.info(f"Monitoring des vérifications toutes les {i_nb_sec_between_check} secondes...")
        while b_success is None:
            # On récupère les vérifications
            d_checks = upload.api_list_checks()
            # On peut déterminer b_success s'il n'y en a plus en attente et en cours
            if len(d_checks["asked"]) == len(d_checks["in_progress"]) == 0:
                b_success = len(d_checks["failed"]) == 0
            # On affiche un rapport via la fonction de callback précisée
            s_message = s_check_message_pattern.format(
                nb_asked=len(d_checks["asked"]),
                nb_in_progress=len(d_checks["in_progress"]),
                nb_passed=len(d_checks["passed"]),
                nb_failed=len(d_checks["failed"]),
            )
            if callback is not None:
                callback(s_message)
            # Si l'état est toujours indéterminé
            if b_success is None:
                # On attend le temps demandé
                time.sleep(i_nb_sec_between_check)
        # On log le dernier rapport selon l'état et on sort
        if b_success:
            Config().om.info(s_message)
            return True
        Config().om.warning(s_message)
        return False

    @staticmethod
    def parse_tree(tree: List[Dict[str, Any]], prefix: str = "") -> Dict[str, int]:
        """Parse l'arborescence renvoyée par l'API en un dictionnaire associant le chemin de chaque fichier à sa taille.
        L'objectif est de permettre de facilement identifier quels sont les fichiers à (re)livrer.

        Args:
            tree (List[Dict[str, Any]]): arborescence à parser
            prefix (str): pré-fixe du chemin

        Returns:
            liste des fichiers envoyés et leur taille
        """
        # Création du dictionnaire pour stocker les fichiers et leur taille
        d_files: Dict[str, int] = {}
        # Parcours de l'arborescence
        for d_element in tree:
            # On complète le chemin
            if prefix != "":
                s_chemin = f"{prefix}/{d_element['name']}"
            else:
                s_chemin = str(d_element["name"])
            # Fichier ou dossier ?
            if d_element["type"].lower() == "file":
                # Fichier, on l'ajoute à notre dictionnaire
                d_files[s_chemin] = int(d_element["size"])
            elif d_element["type"].lower() == "directory":
                # Dossier, on itère dessus avec le nom du dossier comme préfixe
                d_sub_files = UploadAction.parse_tree(d_element["children"], prefix=s_chemin)
                # On fusionne ces fichiers à notre dict principal
                d_files = {**d_files, **d_sub_files}
            else:
                raise GpfSdkError(f"Type d'élément rencontré dans l'arborescence '{d_element['type']}' non géré. Contacter le support.")
        return d_files
